@@ -16,6 +16,7 @@ single.meanvar.poisson.calc <-
       }
       tau=which(tmp==min(tmp,na.rm=T))[1]
       taulike=tmp[tau]
+      tau=tau+minseglen-1 # correcting for the fact that we are starting at minseglen
       if(extrainf==TRUE){
         out=c(tau,null,taulike)
         names(out)=c('cpt','null','alt')
@@ -63,6 +64,8 @@ single.meanvar.poisson<-function(data,penalty="MBIC",pen.value=0,class=TRUE,para
     n=ncol(data)
   }
   if(n<4){stop('Data must have atleast 4 observations to fit a changepoint model.')}
+  if(n<(2*minseglen)){stop('Minimum segment legnth is too large to include a change in this data')}
+  
   pen.value = penalty_decision(penalty, pen.value, n, diffparam=1, asymcheck="meanvar.poisson", method="AMOC")   
   if(is.null(dim(data))==TRUE){
     tmp=single.meanvar.poisson.calc(coredata(data),extrainf=TRUE,minseglen)
@@ -159,12 +162,11 @@ multiple.meanvar.poisson=function(data,mul.method="PELT",penalty="MBIC",pen.valu
   if(!((mul.method=="PELT")||(mul.method=="BinSeg")||(mul.method=="SegNeigh"))){
     stop("Multiple Method is not recognised")
   }
-  if(penalty!="MBIC"){
+  costfunc = "meanvar.poisson"
+  if(penalty=="MBIC"){
     if(mul.method=="SegNeigh"){
       stop('MBIC penalty not implemented for SegNeigh method, please choose an alternative penalty')
     }
-    costfunc = "meanvar.poisson"
-  }else{
     costfunc = "meanvar.poisson.mbic"
   }
   
@@ -176,6 +178,8 @@ multiple.meanvar.poisson=function(data,mul.method="PELT",penalty="MBIC",pen.valu
   else{
     n=ncol(data)
   }
+  if(n<(2*minseglen)){stop('Minimum segment legnth is too large to include a change in this data')}
+  
   pen.value = penalty_decision(penalty, pen.value, n, diffparam=1, asymcheck=costfunc, method=mul.method)
   if(is.null(dim(data))==TRUE){
     # single dataset
