@@ -3,6 +3,6 @@
 	f <- read.dcf(file.path(libname, pkgname, "DESCRIPTION"),
                       c("Version", "Date"))
         packageStartupMessage('Successfully loaded changepoint package version ',
-                              f[1,1],'\n NOTE: Predefined penalty values have changed.  Previous penalty values with a postfix 1 i.e. SIC1 are now without i.e. SIC ', 
+                              f[1,1],'\n NOTE: Predefined penalty values changed in version 2.2.  Previous penalty values with a postfix 1 i.e. SIC1 are now without i.e. SIC ', 
                               'and previous penalties without a postfix i.e. SIC are now with a postfix 0 i.e. SIC0. See NEWS and help files for further details.')
 }
\ No newline at end of file
